/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.tasks;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.InModel;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Library;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.OutModel;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Param;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Superimpose;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3AntToolKit;
import org.eclipse.gmt.am3.tools.ant.toolkit.AM3BuildListener;
import org.eclipse.m2m.atl.adt.launching.AtlVM;
import org.eclipse.m2m.atl.drivers.emf4atl.ASMEMFModel;
import org.eclipse.m2m.atl.engine.AtlLauncher;
import org.eclipse.m2m.atl.engine.AtlModelHandler;
import org.eclipse.m2m.atl.engine.vm.ModelLoader;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;

/**
 * 
 * @author Freddy Allilaire
 * @author Dennis Wagelaar <dennis.wagelaar@vub.ac.be>
 */
public class ATLModelTransformationTask extends Task {

	protected String path;
	protected List inModels = new ArrayList();
	protected List outModels = new ArrayList();
	protected List inoutModels = new ArrayList();
	protected List librarys = new ArrayList();
	private Map params = new HashMap();
	private Map options = new HashMap();
    protected List superimpose = new ArrayList();
	protected Boolean debug;
    protected Boolean allowInterModelReferences;
    protected String vm;
    
    /**
     * @param model
     * @return The ASMModel that corresponds to modelName.
     * @throws BuildException if the model could not be found.
     */
    private ASMModel getModelFromRef(String modelName) throws BuildException {
		if (modelName.startsWith("%")) {
			String modelHandler = modelName.substring(1);
			return AM3AntToolKit.getMOF(getProject(), modelHandler);
		}
		final Object refModel = getProject().getReference(modelName);
		if (refModel instanceof ASMModel) {
			return (ASMModel) refModel;
		} else {
			throw new BuildException("Model '" + modelName + "' was not loaded");
		}
    }
	
	public void execute() throws BuildException {
		AM3AntToolKit.attachLogHandler(getProject());
		
        boolean checkSameModel = true;
        log("Executing ATL transformation " + path);
		
		if (path == null)
			throw new BuildException("path must not be null");
		if (debug == null)
			debug = new Boolean(false);
        if (allowInterModelReferences != null) {
            checkSameModel = !allowInterModelReferences.booleanValue();
        }        

		Map models = new HashMap();
		for (Iterator it = inModels.iterator(); it.hasNext();) {
			InModel currentInModel = (InModel)it.next();
			ASMModel inModel = getModelFromRef(currentInModel.getModel());
			inModel.setIsTarget(false);
			models.put(currentInModel.getName(), inModel);
		}
		for (Iterator it = inoutModels.iterator(); it.hasNext();) {
			InModel currentInModel = (InModel)it.next();
			ASMModel inModel = getModelFromRef(currentInModel.getModel());
			inModel.setIsTarget(true);
			models.put(currentInModel.getName(), inModel);
		}
		for (Iterator it = outModels.iterator(); it.hasNext();) {
			OutModel currentOutModel = (OutModel)it.next();

			if (currentOutModel.getName() == null) {
				throw new BuildException("Out Model '" + currentOutModel + "' must have a field name");
			}
					
			if (currentOutModel.getMetamodel() == null) {
				throw new BuildException("Model '" + currentOutModel.getName() + "' must have a field metamodel");
			}
			
			if (currentOutModel.getModel() == null) {
				throw new BuildException("Model '" + currentOutModel.getName() + "' must have a field model");
			}
			
			ASMModel metamodel = getModelFromRef(currentOutModel.getMetamodel());
			String amhName = AtlModelHandler.getHandlerName(metamodel);
			ModelLoader ml = AM3AntToolKit.getModelLoader(getProject(), amhName);
			ASMModel currentAsmModel = ml.newModel(currentOutModel.getName(), currentOutModel.getPath(), metamodel);
			models.put(currentOutModel.getName(), currentAsmModel);
            if (currentAsmModel instanceof ASMEMFModel)
                ((ASMEMFModel)currentAsmModel).setCheckSameModel(checkSameModel);
			
            String name = currentOutModel.getModel();
			getProject().addReference(name, currentAsmModel);
			getProject().addBuildListener(new AM3BuildListener(name));

		}

		Map libs = new HashMap();
		for (Iterator it = librarys.iterator(); it.hasNext();) {
			Library l = (Library)it.next();
			libs.put(l.getName(), AM3AntToolKit.getURLFromASMFile(l.getPath(), getProject().getBaseDir().getAbsolutePath()));
		}

        List superimps = new ArrayList();
        for (Iterator it = superimpose.iterator(); it.hasNext();) {
            Superimpose s = (Superimpose)it.next();
            superimps.add(AM3AntToolKit.getURLFromASMFile(s.getPath(), getProject().getBaseDir().getAbsolutePath()));
        }
		
		URL asmUrl = AM3AntToolKit.getURLFromASMFile(path, getProject().getBaseDir().getAbsolutePath());
		if(vm == null)
			vm = AtlVM.ATLVM_REGULAR;
		options.put("user.dir", getProject().getBaseDir().getAbsolutePath());
		if (checkSameModel) {
			options.put("checkSameModel", "true");
		} else {
			options.put("checkSameModel", "false");
		}
		if (!debug.booleanValue())
			AtlVM.getVM(vm).launch(asmUrl, libs, models, params, superimps, options);
		else
			AtlLauncher.getDefault().debug(asmUrl, libs, models, params, superimps, options);
	}

	public void addConfiguredLibrary(Library library) {
		librarys.add(library);
	}
	
	public void addConfiguredSuperimpose(Superimpose superimpose) {
		this.superimpose.add(superimpose);
	}
	
	public void addConfiguredInModel(InModel inModel) {
		inModels.add(inModel);
	}
	
	public void addConfiguredOutModel(OutModel outModel) {
		outModels.add(outModel);
	}
	
	public void addConfiguredInoutModel(InModel inoutModel) {
		inoutModels.add(inoutModel);
	}
	
	public void setPath(String path) {
		this.path = path;
	}

	public void setVm(String vm) {
		this.vm = vm;
	}

	public void setDebug(Boolean debug) {
		this.debug = debug;
	}

	public void addConfiguredParam(Param p) {
		params.put(p.getName(), p.getValue());
	}

	public Map getParams() {
		return params;
	}

	public void addConfiguredOption(Param p) {
		options.put(p.getName(), p.getValue());
	}

	public Map getOptions() {
		return options;
	}

	/**
     * @param allowInterModelReferences The allowInterModelReferences to set.
     * @author Dennis Wagelaar <dennis.wagelaar@vub.ac.be>
     */
    public void setAllowInterModelReferences(Boolean allowInterModelReferences) {
        this.allowInterModelReferences = allowInterModelReferences;
    }

    /**
     * @return Returns the allowInterModelReferences.
     * @author Dennis Wagelaar <dennis.wagelaar@vub.ac.be>
     */
    public Boolean getAllowInterModelReferences() {
        return allowInterModelReferences;
    }
}