/*******************************************************************************
 * Copyright (c) 2005 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     INRIA - Initial API and implementation
 *     
 ******************************************************************************/

package org.eclipse.gmt.am3.tools.ant.toolkit;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.StreamHandler;

import org.apache.tools.ant.AntClassLoader;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.eclipse.am3.core.AM3CorePlugin;
import org.eclipse.am3.core.Injectors;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Classpath;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Injector;
import org.eclipse.gmt.am3.tools.ant.tasks.nestedelement.Pathelement;
import org.eclipse.gmt.tcs.extractor.TCSExtractor;
import org.eclipse.gmt.tcs.injector.TCSInjector;
import org.eclipse.m2m.atl.drivers.emf4atl.ASMEMFModelElement;
import org.eclipse.m2m.atl.drivers.emf4atl.EMFModelLoader;
import org.eclipse.m2m.atl.engine.AtlLauncher;
import org.eclipse.m2m.atl.engine.AtlModelHandler;
import org.eclipse.m2m.atl.engine.MarkerMaker;
import org.eclipse.m2m.atl.engine.extractors.xml.XMLExtractor;
import org.eclipse.m2m.atl.engine.injectors.xml.XMLInjector;
import org.eclipse.m2m.atl.engine.vm.ATLVMPlugin;
import org.eclipse.m2m.atl.engine.vm.ModelLoader;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMEnumLiteral;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMOclAny;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMString;
import org.eclipse.m2m.atl.logging.ATLLogFormatter;

/**
 * 
 * @author Freddy Allilaire
 *
 * This toolkit class contains all static method used by AM3 Ant Tasks
 */
public class AM3AntToolKit {

	public static final String EBNFINJECTOR = "ebnf";
	public static final String EBNFEXTRACTOR = "ebnf";
	public static final String XMLINJECTOR = "xml";
	public static final String XMLEXTRACTOR = "xml";
	public static final String XMIEXTRACTOR = "xmi";
	public static final String ATLEXTRACTOR = "atl";
	
	protected static Logger logger = Logger.getLogger(ATLVMPlugin.LOGGER);

	protected static final Handler am3LogHandler = 
		new StreamHandler(System.out, ATLLogFormatter.INSTANCE) {
			public synchronized void publish(LogRecord record) {
				super.publish(record);
				flush();
			}

			public void close() {
				flush();
			}
		};
	
	public static synchronized void attachLogHandler(Project project) {
		if (!project.getBuildListeners().contains(LogHandlerListener.INSTANCE)) {
			logger.fine("Adding AM3 ant log handler");
			am3LogHandler.setLevel(ATLVMPlugin.LOGLEVEL);
			logger.addHandler(am3LogHandler);
			project.addBuildListener(LogHandlerListener.INSTANCE);
		}
	}
	
	//TODO test dynamic model loader
	/**
	 * @param project The current Ant project.
	 * @param modelHandler The model handler ID to use for creating new model loaders.
	 * @return The shared model loader for the given project/amh. Creates a new model loader if necessary.
	 * @author Dennis Wagelaar <dennis.wagelaar@vub.ac.be>
	 */
	public static synchronized ModelLoader getModelLoader(Project project, String modelHandler) {
		final String mlRef = "%modelLoader" + modelHandler;
		ModelLoader ml = (ModelLoader) project.getReference(mlRef);
		if (ml == null) {
			final AtlModelHandler amh = AtlModelHandler.getDefault(modelHandler);
			if (amh != null) {
				ml = amh.createModelLoader();
				project.addReference(mlRef, ml);
				project.addBuildListener(new AM3BuildListener(mlRef));
			}
		}
		return ml;
	}
	
	/**
	 * @param project The current Ant project.
	 * @param modelHandler The model handler ID (EMF/MDR/...).
	 * @return The MOF model for the given project/amh.
	 */
	public static ASMModel getMOF(Project project, String modelHandler) {
		final String amhRef = "%" + modelHandler;
		ASMModel mof = (ASMModel) project.getReference(amhRef);
		if (mof == null) {
			final ModelLoader ml = getModelLoader(project, modelHandler);
			if (ml != null) {
				mof = ml.getMOF();
				project.addReference(amhRef, mof);
			}
		}
		return mof;
	}
	
	private static ASMModel loadModel(String modelHandler, String modelName, ASMModel metamodel, String path, String nsUri, Project project)
	throws CoreException, IOException, BuildException {
		ASMModel ret = null;
		
		final ModelLoader ml = getModelLoader(project, modelHandler);
		if (ml == null) {
			throw new BuildException("Model handler " + modelHandler + " not found.");
		}
		if((ml instanceof EMFModelLoader))
			if(nsUri != null) {
				// TODO delete pre-string 'uri:'
				ret = ml.loadModel(modelName, metamodel, "uri:" + nsUri);
			} else {
				ret = ((EMFModelLoader)ml).loadModel(modelName, metamodel, findFileToURI(path, project));
			}

		else
			ret = ml.loadModel(modelName, metamodel, findFileToInputStream(path, project));

		return ret;
	}
	
	public static URL fileNameToURL(String filePath) throws MalformedURLException {
		if (filePath.startsWith("ext:")) {
			File f = new File(filePath.substring(4));
			return f.toURL();
		}
		else {
			IFile currentLib = findFileInWorkspace(filePath);
			return currentLib.getLocation().toFile().toURL();
		}
	}

	public static URL getURLFromASMFile(String filePath, String basedir) throws BuildException {
		try {
			File foundFile = new File(basedir + '/' + checkAndGetAsmPath(filePath));
			if (foundFile.exists())
				return foundFile.toURL();

			IFile currentAtlFile = findFileInWorkspaceWithErrorThrowing(checkAndGetAsmPath(filePath));
			if (currentAtlFile.exists())
				return findFileInWorkspaceWithErrorThrowing(checkAndGetAsmPath(filePath)).getLocation().toFile().toURL();

		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		throw new BuildException("File with following path: '" + filePath + "' does not exist in the workspace.");
	}

	private static String checkAndGetAsmPath(String filePath) throws BuildException {
		if (filePath.toLowerCase().endsWith(".atl"))
			return filePath.toString().substring(0, filePath.toString().length() - 3) + "asm";
		else if (filePath.toLowerCase().endsWith(".asm"))
			return filePath;
		else
			throw new BuildException("Extension file is not recognized or no corresponding ASM file available");
	}
	
	private static IFile findFileInWorkspaceWithErrorThrowing(String filePath) throws BuildException {
		IFile file = findFileInWorkspace(filePath);
		if (file.exists())
			return file;
		else
			throw new BuildException("File with following path: '" + filePath + "' does not exist in the workspace.");
		
	}
	
	private static IFile findFileInWorkspace(String filePath) throws BuildException {
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();
		
		return wksroot.getFile(new Path(filePath));
	}

	private static URI findFileToURI(String filePath, Project project) throws BuildException {
		if (isAFileSystemPath(filePath, project))
			filePath = convertFileSystemPathToResourcePath(filePath, project);

		if (!isAResourcePath(filePath, project, true))
			throw new BuildException("The path is incorrect.");
			
		IFile file = findFileInWorkspace(filePath);
		if (file.exists())
			return URI.createPlatformResourceURI(filePath, true);
		else
			throw new BuildException("File with following path: '" + filePath + "' does not exist in the workspace.");
	}

	public static String giveUniveralPath(String filePath, Project project) throws BuildException {
		File foundFile = new File(project.getBaseDir().getAbsolutePath() + "/" + filePath);
		return URI.createFileURI(foundFile.getPath()).toString();
	}
	
	private static InputStream findFileToInputStream(String filePath, Project project) throws BuildException {
		File foundFile = new File(project.getBaseDir().getAbsolutePath() + "/" + filePath);
		if (foundFile.exists())
			try {
				return new FileInputStream(foundFile);
			} catch (FileNotFoundException e) {
				throw new BuildException("File with following path: '" + filePath + "' does not exist in the workspace.");
			}
		
		IFile file = findFileInWorkspace(filePath);
		if (file.exists())
			try {
				return file.getContents();
			} catch (CoreException e) {
				e.printStackTrace();
				throw new BuildException("File with following path: '" + filePath + "' does not exist in the workspace.");
			}
		else
			throw new BuildException("File with following path: '" + filePath + "' does not exist in the workspace.");
	} 
	
	private static ASMModel ebnfInjection(String name, String filePath, ModelLoader ml, ASMModel metamodel, Map params, Project project, List classpath) throws BuildException {

		try {
			InputStream in = findFileToInputStream(filePath, project);
			
			ASMModel model = ml.newModel(name, filePath, metamodel);
			TCSInjector ebnfi = new TCSInjector();
			
			ClassLoader oldCl = Thread.currentThread().getContextClassLoader();

			if (classpath != null && classpath.size() == 1 && ((Classpath)classpath.get(0)).getPathElements().size() > 0) {
				List pathElements = ((Classpath)classpath.get(0)).getPathElements();
				
				org.apache.tools.ant.types.Path jarClasspath = new org.apache.tools.ant.types.Path(project);
				for (Iterator it = pathElements.iterator(); it.hasNext();) {
					String cp = ((Pathelement)it.next()).getLocation();
					if (cp == null)
						throw new BuildException("A pathelement must have a location.");

					// TODO
//					boolean existInWorkspace = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(cp)).exists();
//					boolean existInFileSystem = new File(project.getBaseDir().getAbsolutePath() + "/" + cp).exists();

					if (isAResourcePath(cp, project, false))
						cp = convertResourcePathToFileSystemPath(cp, project);
					
					if (isAFileSystemPath(cp, project, false)) {

//					if (existInFileSystem) { 
						org.apache.tools.ant.types.Path tmpPath = new org.apache.tools.ant.types.Path(project, cp);
						jarClasspath.append(tmpPath);
					}
//					else if (existInWorkspace) {
//						org.apache.tools.ant.types.Path tmpPath = new org.apache.tools.ant.types.Path(
//								project,
//								ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(cp)).getLocation().toOSString()
//						);
//						jarClasspath.append(tmpPath);
//					}
					else
						throw new BuildException("Element with path: '"+ cp + "' not found");
				}
				
				AntClassLoader loader = null;
				loader = project.createClassLoader(jarClasspath);
//				loader.setParent(project.getCoreLoader());
				loader.setParent(AM3CorePlugin.getDefault().getLoader());
				loader.setParentFirst(false);
				loader.addJavaLibraries();
				loader.setIsolated(false);
				loader.setThreadContextLoader();
//				loader.forceLoadClass(classname);
				
				Thread.currentThread().setContextClassLoader(loader);
			}
			else if (classpath != null && classpath.size() > 1) {
				throw new BuildException("Only one classpath element is allowed.");
			}
			else 
				Thread.currentThread().setContextClassLoader(AM3CorePlugin.getDefault().getLoader());
			
			ebnfi.inject(model, in, params);
			Thread.currentThread().setContextClassLoader(oldCl);
			
			return model;
		}
//		catch (CoreException e1) {
//			System.err.println(e1);
//		}
		catch (IOException e1) {
			System.err.println(e1);
		}
		
		return null;
	}
	
	public static ASMModel ebnfInjection(String name, String filePath, String metamodelName, ModelLoader ml, Map params, Project project, List classpath) throws BuildException {
		ASMModel metamodel = (ASMModel)project.getReference(metamodelName);
		return ebnfInjection(name, filePath, ml, metamodel, params, project, classpath);
	}
	
	public static ASMModel inject(String name, String filePath, String metamodelName, String modelHandler, Injector injector, Project project) throws BuildException {
		ASMModel ret = null;

		final ModelLoader ml = getModelLoader(project, modelHandler);
		String injectorName = injector.getName(); 
		if(injectorName.equals(AM3AntToolKit.EBNFINJECTOR)) {
			ret = ebnfInjection(name, filePath, metamodelName, ml, injector.getParams(), project, injector.getClasspath());
		} else if(injectorName.equals(AM3AntToolKit.XMLINJECTOR)) {
			ret = xmlInjection(name, filePath, metamodelName, ml, project);
		} else {
			org.eclipse.m2m.atl.engine.injectors.Injector inj = Injectors.getInjector(injectorName);
			if(inj != null) {
				Map paramsMap = new HashMap();
				Map args = injector.getParams();
				Map types = inj.getParameterTypes();
				for(Iterator i = types.keySet().iterator() ; i.hasNext() ; ) {
					String pname = (String)i.next();
					String type = (String)types.get(pname);
					if(type.equals("String")) {
						String val = (String)args.get(pname);
						if(val != null) {
							paramsMap.put(pname, val);
						} else {
							throw new BuildException("Warning: could not find value for parameter \"" + pname + "\" : " + type + ".");
							//logger.warning("could not find value for parameter \"" + pname + "\" : " + type + ".");
						}
					} else if(type.startsWith("Model:")) {
						paramsMap.put(pname, project.getReference((String)args.get(pname)));
//					} else if(type.equals("RandomAccessFile") && (uri != null)) {
//						paramsMap.put(pname, new RandomAccessFile(uri, "r"));
					} else {
						throw new BuildException("Warning: unknown parameter type \"" + type + "\" of \"" + pname + "\".");
					}
				}
				// TODO Fix problem with relative path
				IWorkspace wks = ResourcesPlugin.getWorkspace();
				IWorkspaceRoot wksroot = wks.getRoot();
				
				IFile file = wksroot.getFile(new Path(filePath));
				
				InputStream in;
				try {
					in = file.getContents();
				} catch (CoreException e) {
					throw new BuildException("could not open file", e);
				}
				ASMModel metamodel = (ASMModel)project.getReference(metamodelName);
				ret = ml.newModel(name, filePath, metamodel);
				try {
					inj.inject(ret, in, paramsMap);
				} catch (IOException e) {
					throw new BuildException("injector error", e);
				}
				ret.setIsTarget(false);
			} else {
				throw new BuildException("Unknown injector type: " + injectorName + ".");
			}
		}
	
		return ret;
	}
	
	public static ASMModel loadModel(String name, String modelHandler, String metamodel, String path, String nsUri, Project project) throws BuildException {
		try {			
//			AtlModelHandler amh = AtlModelHandler.getDefault(modelHandler);
			ASMModel asmMetamodel;
			final String amhRef = "%" + modelHandler;
			if (metamodel.equals("MOF") || metamodel.equals(amhRef))
				asmMetamodel = getMOF(project, modelHandler);
			else
				asmMetamodel = (ASMModel)project.getReference(metamodel);

			if (asmMetamodel == null)
				throw new BuildException("Metamodel " + metamodel + " cannot be found.");
			
			return AM3AntToolKit.loadModel(modelHandler, name, asmMetamodel, path, nsUri, project);
		} catch (IOException e) {
			throw new BuildException(e);
		} catch (CoreException e) {
			throw new BuildException(e);
		}
	}
	
	public static void xmlExtraction(final ASMModel model, String path, Project project) {
		try {
			if (isAFileSystemPath(path, project, true))
				path = convertFileSystemPathToResourcePath(path, project, true);

			if (!isAResourcePath(path, project, true))
				throw new BuildException("The path is incorrect.");
			
			IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
			
			PipedInputStream in = new PipedInputStream();
			final OutputStream out = new PipedOutputStream(in);
			
			final Map parameters = Collections.EMPTY_MAP;

			final XMLExtractor xmle = new XMLExtractor();

			new Thread() {
				public void run() {
					try {
						xmle.extract(model, out, parameters);
					} catch(Exception e) {
						e.printStackTrace();
					} finally {
						try {
							out.close();
						} catch(IOException ioe) {
							
						}
					}
				}
			}.start();
			if(file.exists())
				file.setContents(in, IFile.FORCE, null);
			else
				file.create(in, IFile.FORCE, null);
			file.setDerived(true);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}
	
	public static ASMModel xmlInjection(String name, String path, String metamodelName, ModelLoader ml, Project project) {
		try {
			ASMModel metamodel = (ASMModel)project.getReference(metamodelName);
			InputStream in = findFileToInputStream(path, project);
			ASMModel model = ml.newModel(name, path, metamodel);

			XMLInjector xmli = new XMLInjector();

			Map parameters = Collections.EMPTY_MAP;
			xmli.inject(model, in, parameters);	
			
			return model;
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		return null;
	}
	
	public static void ebnfExtraction(final ASMModel model, String path, Map params, Project project)
	throws BuildException {
		try {
			PipedInputStream in = new PipedInputStream();
			final OutputStream out = new PipedOutputStream(in);
						
			final TCSExtractor ebnfe = new TCSExtractor();
			
			Map tempParam = new HashMap();
			for (Iterator it = params.keySet().iterator(); it.hasNext();) {
				String paramName = (String)it.next();
				String paramType = (String)ebnfe.getParameterTypes().get(paramName);
				if (paramType.equals("Model:TCS"))
// TODO
					tempParam.put(paramName, project.getReference((String)params.get(paramName)));
				else if (paramType.equals("String"))
					tempParam.put(paramName, params.get(paramName));
			}

			final Map parameters = tempParam;

			if (isAFileSystemPath(path, project, true))
				path = convertFileSystemPathToResourcePath(path, project, true);

			if (!isAResourcePath(path, project, true))
				throw new BuildException("The path is incorrect.");
			
			IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));			
			
			new Thread() {
				public void run() {
					try {
						ebnfe.extract(model, out, parameters);
					} catch(Exception e) {
						e.printStackTrace();
					} finally {
						try {
							out.close();
						} catch(IOException ioe) {
							
						}
					}
				}
			}.start();
			if(file.exists())
				file.setContents(in, IFile.FORCE, null);
			else
				file.create(in, IFile.FORCE, null);
			file.setDerived(true);
		} catch (IOException ioe) {
			throw new BuildException(ioe);
		} catch (CoreException e) {
			throw new BuildException(e);
		}
	}
	
	public static void atlExtractor(ASMModel model, String path, String queryPath)
	throws BuildException {
		try {
			URL asmUrl = new URL(findFileInWorkspace(queryPath).getFullPath().toString());
			atlExtractor(model, path, asmUrl);
		} catch (MalformedURLException e) {
			throw new BuildException(e);
		}
		
	}
	
	public static void atlExtractor(final ASMModel model, String path, URL asmUrl)
	throws BuildException {
		try {
			Map models = new HashMap();
			models.put(model.getMetamodel().getName(), model.getMetamodel());
			models.put(model.getName(), model);
			Map params = Collections.EMPTY_MAP;
			Map libs = Collections.EMPTY_MAP;
			ASMOclAny s = (ASMOclAny)AtlLauncher.getDefault().launch(asmUrl, libs, models, params, Collections.EMPTY_LIST, Collections.EMPTY_MAP);

			if (s == null)
				return;
			
			if (!(s instanceof ASMString)) {
				throw new BuildException("Query is not an instance of ASMString");
			}
			
			if(((ASMString)s).getSymbol() != null) {
				IFile fileCreated = findFileInWorkspace(path);
				InputStream stream = new ByteArrayInputStream(((ASMString)s).getSymbol().getBytes());
				if (fileCreated.exists()) {
					fileCreated.setContents(stream, true, true, null);
				} else {
					fileCreated.create(stream, true, null);
				}
				fileCreated.setDerived(true);
				try {
					stream.close();
				} catch (IOException e1) {
					throw new BuildException(e1);
				}
			}		
		} catch (CoreException e) {
			throw new BuildException(e);
		}
	}

	public static int applyMarkers(String path, ASMModel pbs, Project project) throws CoreException {
		if (isAFileSystemPath(path, project, true))
			path = convertFileSystemPathToResourcePath(path, project, true);

		if (!isAResourcePath(path, project, true))
			throw new BuildException("The path is incorrect.");
		
		IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
		MarkerMaker markerMaker = new MarkerMaker();

		int nbErrors = 0;
		
		Collection pbsc = pbs.getElementsByType("Problem");
		EObject pbsa[] = new EObject[pbsc.size()];
		int k = 0;
		for(Iterator i = pbsc.iterator() ; i.hasNext() ; ) {
			ASMEMFModelElement ame = (ASMEMFModelElement)i.next();
			pbsa[k] = ame.getObject();
			if("error".equals(((ASMEnumLiteral)ame.get(null, "severity")).getName())) {
				nbErrors++;
			}
			k++;
		}
		markerMaker.resetPbmMarkers(file, pbsa);
		
		return nbErrors;
	}
	
	public static boolean isAFileSystemPath(String path, Project project) {
		return isAFileSystemPath(path, project, false);
	}
	
	public static boolean isAFileSystemPath(String path, Project project, boolean newFile) {
		IResource resource;
		if (Platform.getLocation().equals(path)) {
			resource = ResourcesPlugin.getWorkspace().getRoot();
		} else {
			if (newFile)
				resource = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(path).removeLastSegments(1));
			else
				resource = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(path));
			if (resource == null)
				if (newFile)
					resource = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(project.getBaseDir().getAbsolutePath() + "/" + path).removeLastSegments(1));
				else
					resource = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(project.getBaseDir().getAbsolutePath() + "/" + path));
			if (resource == null || !resource.exists())
				return false;
		}
		return true;
	}
	
	public static boolean isAResourcePath(String path, Project project, boolean newFile) {
		IResource resource = null;
		switch (new Path(path).segmentCount()) {
			case 0 :
				resource = ResourcesPlugin.getWorkspace().getRoot();
				break;
			case 1 :
				resource = ResourcesPlugin.getWorkspace().getRoot().getProject(new Path(path).lastSegment());
				break;
			default :
				resource = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
		}

		if (!newFile)
			return resource.exists();

		if (resource.getLocation() == null)
			return false;

		return true;
	}
	
	public static String convertFileSystemPathToResourcePath(String path, Project project) {
		return convertFileSystemPathToResourcePath(path, project, false);
	}
	
	public static String convertFileSystemPathToResourcePath(String path, Project project, boolean newFile) {
		IResource resource;
		if (Platform.getLocation().equals(path)) {
			resource = ResourcesPlugin.getWorkspace().getRoot();
			return resource.getFullPath().toString();
		} else {
			if (newFile)
				resource = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(path).removeLastSegments(1));
			else
				resource = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(path));
			
			if (resource != null)
				return ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(path)).getFullPath().toString();
			else
				if (newFile)
					resource = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(project.getBaseDir().getAbsolutePath() + "/" + path).removeLastSegments(1));
				else
					resource = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(project.getBaseDir().getAbsolutePath() + "/" + path));
			
			if (resource == null || !resource.exists())
				throw new BuildException("exception.noProjectMatchThePath");
			else
				return ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(project.getBaseDir().getAbsolutePath() + "/" + path)).getFullPath().toString();
		}
		
	}

	public static String convertResourcePathToFileSystemPath(String path, Project project) {
		IResource resource = null;
		switch (new Path(path).segmentCount()) {
			case 0 :
				resource = ResourcesPlugin.getWorkspace().getRoot();
				break;
			case 1 :
				resource = ResourcesPlugin.getWorkspace().getRoot().getProject(new Path(path).lastSegment());
				break;
			default :
				resource = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
		}

		if (resource.getLocation() == null)
			// can occur if the first segment is not a project
			throw new BuildException("exception.pathNotValid");

		return resource.getLocation().toOSString();
	}
}
